﻿namespace PrismAttempt
{
  using System;
  using AbstractionsLibrary;
  using Autofac;
  using Microsoft.Practices.Prism.StoreApps;
  using Microsoft.Practices.Prism.StoreApps.Interfaces;
  using MvvmStart.Services;
  using MvvmStart.ViewModels;
  using PrismAttempt.Abstractions;
  using Windows.ApplicationModel.Activation;

  sealed partial class App : MvvmAppBase
  {
    protected override object Resolve(Type type)
    {
      return (this._container.Resolve(type));
    }
    protected override void OnLaunchApplication(LaunchActivatedEventArgs args)
    {
      this.NavigationService.Navigate("Search", null);
    }
    protected override void OnInitialize(IActivatedEventArgs args)
    {
      base.OnInitialize(args);

      ContainerBuilder builder = new ContainerBuilder();

      builder.
        RegisterInstance<INavigationService>(this.NavigationService).
        As<INavigationService>();

      builder.RegisterType<FlickrService>().As<IFlickrService>();
      builder.RegisterType<PhotoSavingService>().As<IPhotoSavingService>();

      builder.RegisterType<SearchPageViewModel>().AsSelf();
      builder.RegisterType<SearchResultsPageViewModel>().AsSelf();
      builder.RegisterType<PhotoDetailsPageViewModel>().AsSelf();
      builder.RegisterType<SearchResultViewModel>().AsSelf();

      this._container = builder.Build();
    }
    protected override Type GetPageType(string pageToken)
    {
      // My views are called MvvmStart.Views.{0}Page whereas the
      // base class expects something different.
      // Naturally, we wouldn't want to hardcode this namespace
      // right here in the 'real world'.
      string baseName = "MvvmStart.Views.{0}Page";
      string typeName = string.Format(baseName, pageToken);
      Type type = Type.GetType(typeName);
      return (type);
    }
    IContainer _container;
  }
}
