﻿namespace MvvmStart.Services
{
  using System.Collections.Generic;
  using System.IO;
  using System.Linq;
  using System.Net.Http;
  using System.Threading.Tasks;
  using System.Xml.Linq;
  using MvvmStart.Models;
  using PrismAttempt.Abstractions;

  public class FlickrService : IFlickrService
  {
    public async Task<List<FlickrSearchItem>> SearchAsync(string searchTerm)
    {
      List<FlickrSearchItem> resultsList = new List<FlickrSearchItem>();

      XElement xml = await MakeHttpRequest(FlickrUris.MakeSearchUri(searchTerm));

      resultsList =
          (
              from photo in xml.DescendantsAndSelf("photo")
              select new FlickrSearchItem()
              {
                Id = (long)photo.Attribute("id"),
                Title = photo.Attribute("title").Value,
                ImageUrl = MakeImageUrl(
                  (long)photo.Attribute("id"),
                  photo.Attribute("secret").Value,
                  (int)photo.Attribute("farm"),
                  (int)photo.Attribute("server"))
              }
          ).ToList();

      return (resultsList);
    }
    public async Task<string> GetImageTitleAsync(long photoId)
    {
      string title = string.Empty;

      string url = FlickrUris.MakePhotoInfoUrl(photoId.ToString());

      XElement xml = await MakeHttpRequest(url);

      var result = xml.DescendantsAndSelf("title").FirstOrDefault();

      if (result != null)
      {
        title = result.Value;
      }
      return (title);
    }
    public async Task<string> GetMediumSizeImageUrlAsync(long photoId)
    {
      string imageUrl = string.Empty;
      string uri = FlickrUris.MakeSizesPhotoUri(photoId.ToString());

      XElement xml = await MakeHttpRequest(uri);

      var query =
        xml.DescendantsAndSelf("size").Where(
          n => n.Attribute("label").Value.ToLower().Contains("medium"));

      var result = query.FirstOrDefault();

      if (result != null)
      {
        imageUrl = (string)result.Attribute("source");
      }
      return (imageUrl);
    }
    static string MakeImageUrl(long id, string secret, int farm, int server)
    {
      return (string.Format(
      "http://farm{0}.static.flickr.com/{1}/{2}_{3}_m.jpg",
        farm, server, id, secret));
    }
    static async Task<XElement> MakeHttpRequest(string url)
    {
      HttpClient client = new HttpClient();
      XElement xml = null;

      using (HttpResponseMessage response = await client.GetAsync(url))
      {
        using (Stream stream = await response.Content.ReadAsStreamAsync())
        {
          xml = XElement.Load(stream);
        }
      }
      return (xml);
    }
  }
}

