﻿namespace MvvmStart.ViewModels
{
  using System;
  using System.Collections.Generic;
  using System.Threading.Tasks;
  using System.Windows.Input;
  using AbstractionsLibrary;
  using Microsoft.Practices.Prism.StoreApps;
  using Microsoft.Practices.Prism.StoreApps.Interfaces;
  using MvvmStart.Services;
  using PrismAttempt.Abstractions;
  using Windows.UI.Xaml.Navigation;

  public class PhotoDetailsPageViewModel :
    ViewModel
  {
    public PhotoDetailsPageViewModel(
      INavigationService navigationService,
      IPhotoSavingService photoSavingService,
      IFlickrService flickrService)
    {
      this._navigationService = navigationService;
      this._photoSavingService = photoSavingService;
      this._flickrService = flickrService;
    }
    public override void OnNavigatedTo(object navigationParameter,
      NavigationMode navigationMode,
      Dictionary<string, object> viewModelState)
    {
      this.Id = (long)navigationParameter;
      this.GetPhotoDetailsAsync();
    }
    public long Id
    {
      get
      {
        return (this._id);
      }
      set
      {
        base.SetProperty(ref this._id, value);
      }
    }
    public string ImageUrl
    {
      get
      {
        return (this._imageUrl);
      }
      set
      {
        base.SetProperty(ref this._imageUrl, value);
      }
    }
    public string Title
    {
      get
      {
        return (this._title);
      }
      set
      {
        base.SetProperty(ref this._title, value);
      }
    }
    public ICommand BackCommand
    {
      get
      {
        if (this._backCommand == null)
        {
          this._backCommand = new DelegateCommand(this.OnBackCommand);
        }
        return (this._backCommand);
      }
    }
    void OnBackCommand()
    {
      this._navigationService.GoBack();
    }
    public ICommand SaveCommand
    {
      get
      {
        if (this._saveCommand == null)
        {
          this._saveCommand = new DelegateCommand(this.OnSaveCommand);
        }
        return (this._saveCommand);
      }
    }
    void OnSaveCommand()
    {
      this._photoSavingService.SavePhotoToPicturesLibraryAsync(
        new Uri(this.ImageUrl));
    }
    async Task GetPhotoDetailsAsync()
    {
      this.GetTitleAsync();
      this.GetImageUrlAsync();
    }
    async Task GetTitleAsync()
    {
      this.Title = await this._flickrService.GetImageTitleAsync(this.Id);
    }
    async Task GetImageUrlAsync()
    {
      this.ImageUrl = await this._flickrService.GetMediumSizeImageUrlAsync(this.Id);
    }
    IFlickrService _flickrService;
    IPhotoSavingService _photoSavingService;
    INavigationService _navigationService;
    ICommand _backCommand;
    ICommand _saveCommand;
    string _title;
    string _imageUrl;
    long _id;
  }
}
