﻿namespace MvvmStart.ViewModels
{
  using System;
  using System.Collections.Generic;
  using System.Collections.ObjectModel;
  using System.Linq;
  using System.Threading.Tasks;
  using System.Windows.Input;
  using Autofac;
  using Microsoft.Practices.Prism.StoreApps;
  using Microsoft.Practices.Prism.StoreApps.Interfaces;
  using PrismAttempt.Abstractions;
  using Windows.UI.Xaml.Navigation;

  public class SearchResultsPageViewModel :
    ViewModel
  {
    public SearchResultsPageViewModel(
      INavigationService navigationService,
      IFlickrService flickrService,
      Func<SearchResultViewModel> resultViewModelFactory)
    {
      this._navigationService = navigationService;
      this._flickrService = flickrService;
      this._resultViewModelFactory = resultViewModelFactory;
    }
    public override void OnNavigatedTo(object navigationParameter, NavigationMode navigationMode, Dictionary<string, object> viewModelState)
    {
      base.OnNavigatedTo(navigationParameter, navigationMode, viewModelState);
      this.SearchTerm = (string)navigationParameter;
      this.SearchFlickrAsync();
    }
    public string SearchTerm
    {
      get
      {
        return (this._searchTerm);
      }
      set
      {
        base.SetProperty(ref this._searchTerm, value);
      }
    }
    public ICommand BackCommand
    {
      get
      {
        if (this._backCommand == null)
        {
          this._backCommand = new DelegateCommand(this.OnBackCommand);
        }
        return (this._backCommand);
      }
    }
    void OnBackCommand()
    {
      this._navigationService.GoBack();
    }
    public ObservableCollection<SearchResultViewModel> SearchResults
    {
      get
      {
        return (this._searchResults);
      }
      set
      {
        base.SetProperty(ref this._searchResults, value);
      }
    }
    async Task SearchFlickrAsync()
    {
      var searchResults = await
        this._flickrService.SearchAsync(this.SearchTerm);

      this.SearchResults =
        new ObservableCollection<SearchResultViewModel>(
          searchResults.Select(
            modelItem =>
            {
              var viewModel = this._resultViewModelFactory();
              viewModel.SearchItem = modelItem;
              return (viewModel);
            }
      ));
    }
    string _searchTerm;
    ICommand _backCommand;
    ObservableCollection<SearchResultViewModel> _searchResults;
    INavigationService _navigationService;
    IFlickrService _flickrService;
    Func<SearchResultViewModel> _resultViewModelFactory;
  }
}